﻿###############################################################
# Title       :  Insert Students to AD                        #
# Author      :  Eric Krebill                                 #
# Date / Ver  :  10-26-2018 / 1.5                             #
###############################################################

######################### Universal Variables #########################
[xml]$univar = Get-Content -Path $env:HOMEDRIVE\PowerShell\UniVar\UniVar.xml
$insertlogfile = $univar.var.stulog + 'INSERT_AD_log.txt'
$errorlog = $univar.var.stulog + 'INSERT_AD_errors.txt'
$eventlog = $univar.var.stulog + 'INSERT_AD_events.txt'
$passlog = $univar.var.stulog + 'DETAIL_PW_Change_Perm.txt'
$infile = $univar.var.stufile + 'Original_from_Synergy.csv'
$wordList = $univar.var.wordfile + 'wordlist.csv'
$defaultpw = $univar.var.defaultpw
$pwplaceholder = $univar.var.placeholder
$emaildomain = $univar.var.domain
$studentou = $univar.var.stuou
######### Consult README before modifying Universal Variables #########

Import-Module -Name ActiveDirectory

# Set the current date for logging
$currentdate = Get-Date

# Start Current School Year Calculation Function
# Sets the necessary variables for the function
[int]$currentmonth = Get-Date -UFormat %m
[int]$currentyear = Get-Date -UFormat %Y
$yearchange = IF ($currentmonth -le 7) { 0 } ELSE { 1 }
$currentschoolyear = ($currentYear + $yearchange)
# Function sets the grade year correctly regardless of when a student is enrolled
Function Get-GradeYear ([string]$grade)
{
  If ($grade -eq 'K') #If the student is in "K" grade, set the year 0
  {
    $grade = 0
  }
  If ($grade -eq 'DK') #If the student is in "DK" (early childhood), set the year -1
  {
    $grade = -1
  }
  [int]$grade = [convert]::ToInt32($grade, 10)
  [int]($currentschoolyear) - ([int]$grade - 12)
}
# End Current School Year Calculation Function 

# Create temporary password
$passwordTemporary = (ConvertTo-SecureString -AsPlainText -String 'temporaryPasswordHere' -Force)

# Import the password list
$passwordlist = Import-Csv -Path $wordList
$3letters = $passwordlist | Select-Object -ExpandProperty 3letters
$5letters = $passwordlist | Select-Object -ExpandProperty 5letters

IF (Test-Path -Path $infile)
{
  # Import student csv file
  $students = Import-Csv -Path $infile
  
  # Import students from Active Directory
  $adstudents = Get-ADUser -filter "mail -like '*$emaildomain'" -Searchbase $studentou -ResultSetSize $null -Properties sAMAccountName,enabled,division `
    
  # Set all our line counters to zero
  [int]$linecount = 0
  [int]$accountsenabled = 0
  [int]$accountsdisabled = 0
  [int]$accountscreated = 0
  
  FOREACH ($student in $students)
  {
    # Checks to see if the user already exists in Active Directory.  If yes, skip.  If no, create.
    IF (!(Get-ADUser -Filter "SamAccountName -eq $($student.SIS_NUMBER)")) 
    {
      IF (!($student.SIS_NUMBER -eq 'SIS_NUMBER')) 
      {
        # Generates random password - 3 letter word + 5 letter word
        $pass3 = Get-Random -InputObject $3letters -Count 1
        $pass5 = Get-Random -InputObject $5letters -Count 1

        # Set static password - for use when you're using a default password
        $staticpass = $defaultpw

        # Splits out Instructor's Name since we just need last name
        $instructor,$junk = $student.HR_CLASS.split(',')

        # Sets the per-user grade level using the calculations above
        [string]$gradelevel = $student.GRADE_LEVEL
        $gradyear = Get-GradeYear $gradelevel

        ##### Standard AD Account Variables from CSV
        $sAMAccountName = $student.SIS_NUMBER
        $logonName = $student.SIS_NUMBER + '@domain.local'
        $givenName = $student.FIRST_NAME -replace '\*'
        $sn = $student.LAST_NAME -replace '\*'
        $displayName = $givenName + ' ' + $sn
        $mail = $sAMAccountName + $emaildomain
        $description = $gradyear
        $department = '/Students/' + $gradyear
        $ouPath = 'OU=' + $gradyear + ',' + $studentOU
        
        ##### Dynamic account variables depending on grade level
        IF (($student.GRADE_LEVEL -lt '06') -or ($student.GRADE_LEVEL -eq 'K') -or ($student.GRADE_LEVEL -eq 'DK'))
        {
          $password = $pass3 + $pass5
          $passwordSecure = (ConvertTo-SecureString -AsPlainText -String $password -Force)
          $Division = $pass3 + $pass5
          $company = $instructor # If elementary student, output teacher's last name
        }
        ELSE
        {
          $password = $staticpass
          $passwordSecure = (ConvertTo-SecureString -AsPlainText -String $password -Force)
          $Division = $pwplaceholder # The placeholder shows that lookup is not available on the lookup site.
          $company = 'MS/HS' # If Middle / High School student, put in placeholder
        }

        ##### rocket group - This doesn't store in AD, the student gets added to this group so they get the proper filtering
        $sgroup1 = switch ($student.GRADE_LEVEL) 
        { 
          { 'DK', 'K', '01', '02' -contains $_ } {'rocket_group1'}
          { '03', '04', '05' -contains $_ } {'rocket_group2'} 
          { '06', '07', '08' -contains $_ } {'rocket_group3'} 
          { '09', '10', '11', '12' -contains $_ } {'rocket_group4'} 
          default {'rocket_group1'}
        }

        # Logging any students who were found
        Add-Content -Path $insertlogfile -Value "$currentdate - New user found: $displayName - $sAMAccountName"
    
        # Define new user attributes for splatting
        $newUserSplat = @{
          Name = $displayName
          DisplayName = $displayName
          GivenName = $givenName
          Surname = $sn
          SamAccountNAme = $samAccountName
          UserPrincipalName = $logonName
          EmailAddress = $mail
          AccountPassword = $passwordTemporary
          Division = $Division
          ChangePasswordAtLogon = $false
          CannotChangePassword = $false
          PasswordNeverExpires = $true
          Path = $ouPath
          Company = $company
          Department = $department
          Description = $description
          Enabled = $true
        }

        TRY
        {
          # Create the user
          New-ADUser @newUserSplat

          # Pull dynamic group set above
          Add-ADGroupMember -identity $sgroup1 -Members $sAMAccountName -ErrorAction Stop

          # Place static security groups that all students should have here.  Comment (#) one or both of these IF you don't need them.
          Add-ADGroupMember -identity 'Google Email' -Members $sAMAccountName -ErrorAction Stop
          Add-ADGroupMember -identity 'students' -Members $sAMAccountName -ErrorAction Stop
          
          # Set the real password
          Set-ADAccountPassword -Identity $samAccountName -Reset -NewPassword $passwordSecure
          
          # Set password change permissions
          IF ($password -eq 'Lookup_Not_Available')
          {
            Set-ADUser -identity $sAMAccountName -CannotChangePassword $false -ErrorAction Continue
            Add-Content -Path $passlog -Value "$(Get-Date) - Set $sAMAccountName Cannot Change Password = False" -PassThru
          }
          ELSE
          {
            Set-ADUser -identity $sAMAccountName -CannotChangePassword $true -ErrorAction Continue
            Add-Content -Path $passlog -Value "$(Get-Date) - Set $sAMAccountName Cannot Change Password = True" -PassThru
          }

          Add-Content -Path $eventlog -Value "$sAMAccountName - $description - was created on $currentdate"
          $accountscreated++
          $linecount++
        }
        CATCH
        {
          $errorUser = $samAccountName
          $linecount++
          Add-Content -Path $insertlogfile -Value "$currentdate::AD import::ERROR::There was an error with adding / setting $errorUser in active directory at line $linecount. Check $errorlog"
          Add-Content -Value $error[0] -Path $errorlog
          Add-Content -Value $user -Path $errorlog
        }
      }  
    }

  }
  
  # Enable active and disable inactive students
  FOREACH ($adstudent in $adstudents)
  {
    $found = $false
    FOREACH ($student in $students)
    {        
      IF ($adstudent.sAMAccountName -eq $student.SIS_NUMBER)
      {
        $found = $true
        IF ($($adstudent.enabled) -eq $false)
        {
          Enable-ADAccount $adstudent -ErrorAction Stop
          $accountsenabled++
        }
      }
    }
    IF (!$found)
    {
      IF ($($adstudent.enabled) -eq $true)
      {
        #Disable-ADAccount $adstudent -ErrorAction Stop
        $accountsdisabled++
      }
    }
  }
  Add-Content -Path $insertlogfile -Value "$currentdate - $accountscreated created::$accountsenabled enabled::$accountsdisabled disabled"
}

# Time to cleanup
Remove-Item -Path $infile -Force
